@@ -32,12 +32,16 @@
 #include <assert.h>
 #include "igzip_lib.h"
 
+#if __x86_64__  || __i386__ || _M_X64 || _M_IX86
 #ifdef _MSC_VER
 # include <intrin.h>
 # define inline __inline
 #else
 # include <x86intrin.h>
 #endif
+#else
+# define inline __inline
+#endif //__x86_64__  || __i386__ || _M_X64 || _M_IX86
 
 static inline uint32_t bsr(uint32_t val)
 {
@@ -51,18 +55,16 @@ static inline uint32_t bsr(uint32_t val)
 	return msb;
 }
 
-static inline uint32_t tzcnt(uint64_t val)
+static inline uint32_t tzbytecnt(uint64_t val)
 {
 	uint32_t cnt;
 
 #ifdef __BMI__
 	cnt = __tzcnt_u64(val);
 	cnt = cnt / 8;
-#elifdef __x86_64__
+#elif defined(__x86_64__)
 
-	cnt = __bsfq(val);
-	if(val == 0)
-		cnt = 64;
+	cnt = (val == 0)? 64 : __builtin_ctzll(val);
 	cnt = cnt / 8;
 
 #else
@@ -204,7 +206,7 @@ static inline uint32_t compute_hash_mad(uint32_t data)
 	int16_t data_low;
 	int16_t data_high;
 
-	data_low = data;        ;
+	data_low = data;
 	data_high = data >> 16;
 	data = PROD1 * data_low + PROD2 * data_high;
 
@@ -241,7 +243,7 @@ static inline int compare258(uint8_t * str1, uint8_t * str2, uint32_t max_length
 		test = *(uint64_t *) str1;
 		test ^= *(uint64_t *) str2;
 		if(test != 0)
-			return count + tzcnt(test);
+			return count + tzbytecnt(test);
 		str1 += 8;
 		str2 += 8;
 	}
