@@ -182,8 +182,10 @@ void create_rand_repeat_data(uint8_t * data, int size)
 
 	max_repeat_data += power_of_2_array[power];
 
-	if (size-- > 0)
+	if (size > 0) {
+		size--;
 		*data++ = rand();
+	}
 
 	while (size > 0) {
 		next_data = rand() % max_repeat_data;
@@ -208,8 +210,10 @@ void create_rand_repeat_data(uint8_t * data, int size)
 					*data = *(data - distance);
 					data++;
 				}
-			} else
+			} else {
 				memcpy(data, data - distance, length);
+				data += length;
+			}
 		}
 	}
 }
@@ -656,11 +660,32 @@ int inflate_multi_pass(uint8_t * compress_buf, uint64_t compress_len,
 				state->next_out = uncomp_tmp;
 			}
 		}
+#ifdef VERBOSE
+		printf("Pre inflate\n");
+		printf
+		    ("compressed_size = 0x%05x, in_processed = 0x%05x, in_size = 0x%05x, avail_in = 0x%05x\n",
+		     compress_len, comp_processed, comp_tmp_size, state->avail_in);
+		printf
+		    ("data_size       = 0x%05x, out_processed  = 0x%05x, out_size  = 0x%05x, avail_out  = 0x%05x, total_out  = 0x%05x\n",
+		     *uncompress_len, uncomp_processed, uncomp_tmp_size, state->avail_out,
+		     state->total_out);
+#endif
 
 		ret = isal_inflate_with_checks(state, compress_len, *uncompress_len, comp_tmp,
 					       comp_tmp_size, comp_processed, uncomp_tmp,
 					       uncomp_tmp_size, uncomp_processed);
 
+#ifdef VERBOSE
+		printf("Post inflate\n");
+		printf
+		    ("compressed_size = 0x%05x, in_processed = 0x%05x, in_size = 0x%05x, avail_in = 0x%05x\n",
+		     compress_len, comp_processed, comp_tmp_size, state->avail_in);
+		printf
+		    ("data_size       = 0x%05x, out_processed  = 0x%05x, out_size  = 0x%05x, avail_out  = 0x%05x, total_out  = 0x%05x\n",
+		     *uncompress_len, uncomp_processed, uncomp_tmp_size, state->avail_out,
+		     state->total_out);
+#endif
+
 		if (state->block_state == ISAL_BLOCK_FINISH || ret != 0) {
 			memcpy(uncompress_buf + uncomp_processed, uncomp_tmp, uncomp_tmp_size);
 			*uncompress_len = state->total_out;
@@ -2282,8 +2307,10 @@ int test_compress_file(char *file_name)
 	FILE *in_file = NULL;
 
 	in_file = fopen(file_name, "rb");
-	if (!in_file)
+	if (!in_file) {
+		printf("Failed to open file %s\n", file_name);
 		return FILE_READ_FAILED;
+	}
 
 	in_size = get_filesize(in_file);
 	if (in_size > MAX_FILE_SIZE)
@@ -2291,8 +2318,10 @@ int test_compress_file(char *file_name)
 
 	if (in_size != 0) {
 		in_buf = malloc(in_size);
-		if (in_buf == NULL)
+		if (in_buf == NULL) {
+			printf("Failed to allocate in_buf for test_compress_file\n");
 			return MALLOC_FAILED;
+		}
 		fread(in_buf, 1, in_size, in_file);
 	}
 
